////////////////////////////////////////////////////////////////
// rotatingHelixFPS.cpp
//
// This program enhances rotatingHelix2.cpp to add a routine to
// count and output the number of frames drawn per second to
// the debug window.
//
// Interaction:
// Press space to toggle between animation on and off.
// 
// Sumanta Guha.
//////////////////////////////////////////////////////////////// 

#define _USE_MATH_DEFINES 

#include <cstdlib>
#include <cmath>
#include <iostream>

#include <GL/glew.h>
#include <GL/freeglut.h> 

using namespace std;

// Globals.
static int isAnimate = 0; // Animated?
static float angle = 0.0; // Angle of rotation.
static int frameCount = 0; // Number of frames

// Drawing routine.
void drawScene(void)
{  
   float R = 20.0; // Radius of helix.

   float t; // Angle parameter along helix.

   frameCount++; // Increment number of frames every redraw.

   glClear(GL_COLOR_BUFFER_BIT);
   glColor3f(0.0, 0.0, 0.0);
   glPushMatrix();

   // The Trick: to align the axis of the helix along the y-axis prior to
   // rotation and then return it to its original location.
   glTranslatef(0.0, 0.0, -60.0);
   glRotatef(angle, 0.0, 1.0, 0.0);
   glTranslatef(0.0, 0.0, 60.0);

   glBegin(GL_LINE_STRIP);
   for(t = -10 * M_PI; t <= 10 * M_PI; t += M_PI/20.0) 
      glVertex3f(R * cos(t), t, R * sin(t) - 60.0);
   glEnd();
   
   glPopMatrix();
   glutSwapBuffers();
}

// Initialization routine.
void setup(void) 
{
   glClearColor(1.0, 1.0, 1.0, 0.0); 
}

// OpenGL window reshape routine.
void resize(int w, int h)
{
   glViewport(0, 0, w, h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum(-5.0, 5.0, -5.0, 5.0, 5.0, 100.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

// Routine to increase the rotation angle.
void increaseAngle(void)
{
   angle += 5.0;
   if (angle > 360.0)
       angle -= 360.0;
   glutPostRedisplay();
}

// Routine to count the number of frames drawn every second.
void frameCounter(int value)
{
   // No output the first time frameCounter() is called (from main()).
   if (value != 0)
      cout << "FPS = " << frameCount << endl;
   frameCount = 0;
   glutTimerFunc(1000, frameCounter, 1);
}

// Keyboard input processing routine.
void keyInput(unsigned char key, int x, int y)
{
   switch(key) 
   {
      case 27:
         exit(0);
         break;
      case ' ':
         if(isAnimate) 
         {
            isAnimate = 0;
            glutIdleFunc(NULL);
         }
         else 
         {
            isAnimate = 1;
            glutIdleFunc(increaseAngle);
         }
         break;
      default:
         break;
   }
}

// Routine to output interaction instructions to the C++ window.
void printInteraction(void)
{
   cout << "Interaction:" << endl;
   cout << "Press space to toggle between animation on and off." << endl;
}

// Main routine.
int main(int argc, char **argv) 
{
   printInteraction();
   glutInit(&argc, argv);

   glutInitContextVersion(3, 1);
   glutInitContextProfile(GLUT_COMPATIBILITY_PROFILE); 

   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA); 
   glutInitWindowSize(500, 500);
   glutInitWindowPosition(100, 100); 
   glutCreateWindow("rotatingHelixFPS.cpp");
   glutDisplayFunc(drawScene); 
   glutReshapeFunc(resize);  
   glutKeyboardFunc(keyInput);
   glutTimerFunc(0, frameCounter, 0); // Initial call of frameCounter().

   glewExperimental = GL_TRUE; 
   glewInit(); 
   
   setup(); 

   glutMainLoop(); 
}
